import json
from uuid import UUID

import pytest

from kiota_serialization_json.json_parse_node import JsonParseNode
from kiota_serialization_json.json_serialization_writer import JsonSerializationWriter

from ..helpers import OfficeLocation, User, User2
from ..helpers.intersection_type import InterSectionType

display_name = "John Doe"


def test_parse_intersection_type_complex_property1():
    json_string = '{"id": "8f841f30-e6e3-439a-a812-ebd369559c36",\
        "display_name": "John Doe", "age": 32, "gpa": 3.9, "office_location": "dunhill"}'

    parse_node = JsonParseNode(json.loads(json_string))
    result = parse_node.get_object_value(InterSectionType)

    assert result.composed_type1
    assert result.composed_type2
    assert not result.composed_type3
    assert not result.string_value
    assert result.composed_type1.id == UUID("8f841f30-e6e3-439a-a812-ebd369559c36")
    assert result.composed_type1.office_location == OfficeLocation.Dunhill
    assert result.composed_type2.display_name == display_name
    assert result.composed_type2.age == 32


def test_parse_intersection_type_complex_property2():
    json_string = '{"id": 43, "display_name": "John Doe", "age": 32, "gpa": 3.9,\
        "office_location": "oval"}'

    parse_node = JsonParseNode(json.loads(json_string))
    result = parse_node.get_object_value(InterSectionType)

    assert result.composed_type1
    assert result.composed_type2
    assert not result.composed_type3
    assert not result.string_value
    assert not result.composed_type1.id
    assert result.composed_type1.office_location == OfficeLocation.Oval
    assert not result.composed_type2.id
    assert result.composed_type2.display_name == display_name
    assert result.composed_type2.age == 32


def test_parse_intersection_type_complex_property3():
    json_string = '[{"id": "8f841f30-e6e3-439a-a812-ebd369559c36",\
        "display_name": "John Doe", "age": 32, "gpa": 3.9, "office_location": "dunhill"},\
            {"id": "8f841f30-e6e3-439a-a812-ebd369559c36", "display_name": "Jane Doe",\
                "age": 30, "gpa": 3.8, "office_location": "oval"}]'

    parse_node = JsonParseNode(json.loads(json_string))
    result = parse_node.get_object_value(InterSectionType)

    assert not result.composed_type1
    assert not result.composed_type2
    assert result.composed_type3
    assert not result.string_value
    assert len(result.composed_type3) == 2
    assert result.composed_type3[1].office_location == OfficeLocation.Oval
    assert result.composed_type3[0].office_location == OfficeLocation.Dunhill


def test_parse_intersection_type_string_value():
    json_string = json.dumps(display_name)
    parse_node = JsonParseNode(json.loads(json_string))
    result = parse_node.get_object_value(InterSectionType)

    assert not result.composed_type1
    assert not result.composed_type2
    assert not result.composed_type3
    assert result.string_value
    assert result.string_value == display_name


def test_serializes_intersection_type_string_value():
    writer = JsonSerializationWriter()
    model = InterSectionType(string_value=display_name)
    model.serialize(writer)

    content = writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == json.dumps(display_name)


def test_serializes_intersection_type():
    writer = JsonSerializationWriter()
    composed_type1 = User(
        id=UUID("8f841f30-e6e3-439a-a812-ebd369559c36"), office_location=OfficeLocation.Dunhill
    )
    composed_type2 = User2(display_name=display_name, age=32)
    model = InterSectionType(composed_type1=composed_type1, composed_type2=composed_type2)
    model.serialize(writer)

    content = writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == '{"id": "8f841f30-e6e3-439a-a812-ebd369559c36", '\
        '"office_location": "dunhill", "display_name": "John Doe", "age": 32}'


def test_serializes_intersection_type_complex_property2():
    writer = JsonSerializationWriter()
    composed_type2 = User2(id=1, display_name=display_name, age=32)
    model = InterSectionType(composed_type2=composed_type2)
    model.serialize(writer)

    content = writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == '{"id": 1, "display_name": "John Doe", "age": 32}'


def test_serializes_intersection_type_complex_property3():
    writer = JsonSerializationWriter()
    user_1 = User(
        id=UUID("8f841f30-e6e3-439a-a812-ebd369559c36"),
        office_location=OfficeLocation.Dunhill,
        is_active=True
    )
    user_2 = User(
        id=UUID("7031ea06-d26d-4c9a-a738-ce6fe524f05f"),
        office_location=OfficeLocation.Oval,
        is_active=False
    )
    model = InterSectionType(composed_type3=[user_1, user_2])
    model.serialize(writer)

    content = writer.get_serialized_content()
    content_string = content.decode('utf-8')
    assert content_string == '[{"id": "8f841f30-e6e3-439a-a812-ebd369559c36", '\
        '"office_location": "dunhill", "is_active": true}, '\
            '{"id": "7031ea06-d26d-4c9a-a738-ce6fe524f05f", "office_location": "oval", '\
                '"is_active": false}]'



from __future__ import annotations

import json
import warnings
from datetime import date, datetime, time, timedelta
from enum import Enum
from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar, Union
from uuid import UUID

import pendulum
import re

from kiota_abstractions.serialization import Parsable, ParsableFactory, ParseNode

T = TypeVar("T")

U = TypeVar("U", bound=Parsable)

K = TypeVar("K", bound=Enum)


class JsonParseNode(ParseNode, Generic[T, U]):

    def __init__(self, node: Any) -> None:
        """
        Args:
            node (Any): The JsonElement\
                to initialize the node with
        """
        self._json_node = node
        self._on_before_assign_field_values: Optional[Callable[[Parsable], None]] = None
        self._on_after_assign_field_values: Optional[Callable[[Parsable], None]] = None

    def get_child_node(self, identifier: str) -> Optional[ParseNode]:
        """Gets a new parse node for the given identifier
        Args:
            identifier (str): The identifier of the current node property
        Returns:
            Optional[ParseNode]: A new parse node for the given identifier
        """
        if not identifier:
            raise ValueError("identifier cannot be None or empty.")

        if isinstance(node := self._json_node, dict) and identifier in node:
            return self._create_new_node(node[identifier])
        return None

    def get_str_value(self) -> Optional[str]:
        """Gets the string value from the json node
        Returns:
            str: The string value of the node
        """
        return self._json_node if isinstance(self._json_node, str) else None

    def get_bool_value(self) -> Optional[bool]:
        """Gets the boolean value of the json node
        Returns:
            bool: The boolean value of the node
        """
        return self._json_node if isinstance(self._json_node, bool) else None

    def get_int_value(self) -> Optional[int]:
        """Gets the integer value of the json node
        Returns:
            int: The integer value of the node
        """
        return self._json_node if isinstance(self._json_node, int) else None

    def get_float_value(self) -> Optional[float]:
        """Gets the float value of the json node
        Returns:
            float: The number value of the node
        """
        return float(self._json_node) if isinstance(self._json_node, (float, int)) else None

    def get_uuid_value(self) -> Optional[UUID]:
        """Gets the UUID value of the json node
        Returns:
            UUID: The GUID value of the node
        """
        if isinstance(self._json_node, UUID):
            return self._json_node
        if isinstance(self._json_node, str):
            return UUID(self._json_node)
        return None

    def get_datetime_value(self) -> Optional[datetime]:
        """Gets the datetime value of the json node
        Returns:
            datetime: The datetime value of the node
        """
        if isinstance(self._json_node, datetime):
            return self._json_node

        if isinstance(self._json_node, str):
            if len(self._json_node) < 10:
                return None

            datetime_obj = pendulum.parse(self._json_node, exact=True)
            if isinstance(datetime_obj, pendulum.DateTime):
                return datetime_obj
        return None

    def get_timedelta_value(self) -> Optional[timedelta]:
        """Gets the timedelta value of the node
        Returns:
            timedelta: The timedelta value of the node
        """
        if isinstance(self._json_node, timedelta):
            return self._json_node
        if isinstance(self._json_node, str):
            datetime_obj = pendulum.parse(self._json_node, exact=True)
            if isinstance(datetime_obj, pendulum.Duration):
                return datetime_obj.as_timedelta()
        return None

    def get_date_value(self) -> Optional[date]:
        """Gets the date value of the node
        Returns:
            date: The datevalue of the node in terms on year, month, and day.
        """
        if isinstance(self._json_node, date):
            return self._json_node
        if isinstance(self._json_node, str):
            datetime_obj = pendulum.parse(self._json_node, exact=True)
            if isinstance(datetime_obj, pendulum.Date):
                return datetime_obj
        return None

    def get_time_value(self) -> Optional[time]:
        """Gets the time value of the node
        Returns:
            time: The time value of the node in terms of hour, minute, and second.
        """
        if isinstance(self._json_node, time):
            return self._json_node
        if isinstance(self._json_node, str):
            datetime_obj = pendulum.parse(self._json_node, exact=True)
            if isinstance(datetime_obj, pendulum.Time):
                return datetime_obj
        return None

    def get_collection_of_primitive_values(self, primitive_type: Any) -> Optional[List[T]]:
        """Gets the collection of primitive values of the node
        Returns:
            List[T]: The collection of primitive values
        """

        primitive_types = {bool, str, int, float, UUID, datetime, timedelta, date, time, bytes}

        def func(item):
            generic_type = primitive_type if primitive_type else type(item)
            current_parse_node = self._create_new_node(item)
            if generic_type in primitive_types:
                method = getattr(current_parse_node, f'get_{generic_type.__name__.lower()}_value')
                return method()
            raise Exception(f"Encountered an unknown type during deserialization {generic_type}")

        if isinstance(self._json_node, str):
            return list(map(func, json.loads(self._json_node)))
        return list(map(func, list(self._json_node)))

    def get_collection_of_object_values(self, factory: ParsableFactory) -> List[U]:
        """Gets the collection of type U values from the json node
        Returns:
            List[U]: The collection of model object values of the node
        """
        if isinstance(self._json_node, list):
            return list(
                map(
                    lambda x: self._create_new_node(x).get_object_value(factory),  # type: ignore
                    self._json_node,
                )
            )
        return []

    def get_collection_of_enum_values(self, enum_class: K) -> List[Optional[K]]:
        """Gets the collection of enum values of the json node
        Returns:
            List[K]: The collection of enum values
        """
        if isinstance(self._json_node, list):
            return list(
                map(lambda x: self._create_new_node(x).get_enum_value(enum_class), self._json_node)
            )
        return []

    def get_enum_value(self, enum_class: K) -> Optional[K]:
        """Gets the enum value of the node
        Returns:
            Optional[K]: The enum value of the node
        """
        raw_key = str(self._json_node)
        if not raw_key:
            return None

        camel_case_key = None
        if raw_key.lower() == "none":
            # None is a reserved keyword in python
            camel_case_key = "None_"
        else:
            camel_case_key = raw_key[0].upper() + raw_key[1:]

        try:
            return enum_class[camel_case_key]  # type: ignore
        except KeyError:
            return None

    def get_object_value(self, factory: ParsableFactory) -> U:
        """Gets the model object value of the node
        Returns:
            Parsable: The model object value of the node
        """

        result = factory.create_from_discriminator_value(self)
        if on_before := self.on_before_assign_field_values:
            on_before(result)
        self._assign_field_values(result)
        if on_after := self.on_after_assign_field_values:
            on_after(result)
        return result

    def get_bytes_value(self) -> Optional[bytes]:
        """Get a bytearray value from the nodes
        Returns:
            bytearray: The bytearray value from the nodes
        """
        base64_string = str(self._json_node)
        if not base64_string:
            return None
        return base64_string.encode("utf-8")

    @property
    def on_before_assign_field_values(self) -> Optional[Callable[[Parsable], None]]:
        """Gets the callback called before the node is deserialized.
        Returns:
            Callable[[Parsable], None]: the callback called before the node is deserialized.
        """
        return self._on_before_assign_field_values

    @on_before_assign_field_values.setter
    def on_before_assign_field_values(self, value: Callable[[Parsable], None]) -> None:
        """Sets the callback called before the node is deserialized.
        Args:
            value (Callable[[Parsable], None]): the callback called before the node is
            deserialized.
        """
        self._on_before_assign_field_values = value

    @property
    def on_after_assign_field_values(self) -> Optional[Callable[[Parsable], None]]:
        """Gets the callback called before the node is deserialized.
        Returns:
            Callable[[Parsable], None]: the callback called before the node is deserialized.
        """
        return self._on_after_assign_field_values

    @on_after_assign_field_values.setter
    def on_after_assign_field_values(self, value: Callable[[Parsable], None]) -> None:
        """Sets the callback called after the node is deserialized.
        Args:
            value (Callable[[Parsable], None]): the callback called after the node is
            deserialized.
        """
        self._on_after_assign_field_values = value

    def _assign_field_values(self, item: U) -> None:
        """Assigns the field values to the model object"""

        # if object is null
        if not isinstance(self._json_node, dict):
            return

        item_additional_data = None
        if not hasattr(item, "additional_data") or item.additional_data is None:
            item_additional_data = {}
        else:
            item_additional_data = item.additional_data

        field_deserializers = item.get_field_deserializers()

        for field_name, field_value in self._json_node.items():
            if field_name in field_deserializers:
                if field_value is None:
                    continue
                field_deserializer = field_deserializers[field_name]
                field_deserializer(JsonParseNode(field_value))
            elif item_additional_data is not None:
                item_additional_data[field_name] = self.try_get_anything(field_value)
            else:
                warnings.warn(
                    f"Found additional property {field_name} to \
                    deserialize but the model doesn't support additional data"
                )

    def __is_four_digit_number(self, value: str) -> bool:
        pattern = r'^\d{4}$'
        return bool(re.match(pattern, value))

    def try_get_anything(self, value: Any) -> Any:
        if isinstance(value, (int, float, bool)) or value is None:
            return value
        if isinstance(value, list):
            return list(map(self.try_get_anything, value))
        if isinstance(value, dict):
            return dict(map(lambda x: (x[0], self.try_get_anything(x[1])), value.items()))
        if isinstance(value, str):
            try:
                if self.__is_four_digit_number(value):
                    return value

                datetime_obj = pendulum.parse(value)
                if isinstance(datetime_obj, pendulum.Duration):
                    return datetime_obj.as_timedelta()
                return datetime_obj
            except ValueError:
                pass
            try:
                return UUID(value)
            except:
                pass
            return value
        raise ValueError(f"Unexpected additional value type {type(value)} during deserialization.")

    def _create_new_node(self, node: Any) -> JsonParseNode:
        new_node: JsonParseNode = JsonParseNode(node)
        new_node.on_before_assign_field_values = self.on_before_assign_field_values
        new_node.on_after_assign_field_values = self.on_after_assign_field_values
        return new_node



from __future__ import annotations

from dataclasses import dataclass
from typing import Callable, Dict, Optional
from uuid import UUID

from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter


@dataclass
class Entity(Parsable):

    id: Optional[UUID] = None

    @staticmethod
    def create_from_discriminator_value(parse_node: Optional[ParseNode] = None) -> Entity:
        """
        Creates a new instance of the appropriate class based on discriminator value
        Args:
            parseNode: The parse node to use to read the discriminator value and create the object
        Returns: Attachment
        """
        if not parse_node:
            raise ValueError("parse_node cannot be undefined")
        return Entity()

    def get_field_deserializers(self) -> Dict[str, Callable[[ParseNode], None]]:
        """Gets the deserialization information for this object.

        Returns:
            Dict[str, Callable[[ParseNode], None]]: The deserialization information for this
            object where each entry is a property key with its deserialization callback.
        """
        return {
            "id": lambda n: setattr(self, 'id', n.get_uuid_value()),
        }

    def serialize(self, writer: SerializationWriter) -> None:
        """Writes the objects properties to the current writer.

        Args:
            writer (SerializationWriter): The writer to write to.
        """
        if not writer:
            raise ValueError("Writer cannot be undefined")
        writer.write_uuid_value("id", self.id)



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import date, datetime
from typing import Any, Callable, Dict, List, Optional, TypeVar
from uuid import UUID

from kiota_abstractions.serialization import (
    AdditionalDataHolder,
    Parsable,
    ParseNode,
    SerializationWriter,
)

from .office_location import OfficeLocation

T = TypeVar('T')


@dataclass
class User(Parsable, AdditionalDataHolder):
    additional_data: Dict[str, Any] = field(default_factory=dict)
    id: Optional[UUID] = None
    office_location: Optional[OfficeLocation] = None
    updated_at: Optional[datetime] = None
    birthday: Optional[date] = None
    business_phones: Optional[List[str]] = None
    mobile_phone: Optional[str] = None
    is_active: Optional[bool] = None

    @staticmethod
    def create_from_discriminator_value(parse_node: Optional[ParseNode] = None) -> User:
        """
        Creates a new instance of the appropriate class based on discriminator value
        Args:
            parseNode: The parse node to use to read the discriminator value and create the object
        Returns: Attachment
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null")
        return User()

    def get_field_deserializers(self) -> Dict[str, Callable[[ParseNode], None]]:
        """Gets the deserialization information for this object.

        Returns:
            Dict[str, Callable[[ParseNode], None]]: The deserialization information for this
            object where each entry is a property key with its deserialization callback.
        """
        return {
            "id":
            lambda n: setattr(self, 'id', n.get_uuid_value()),
            "office_location":
            lambda n: setattr(self, 'office_location', n.get_enum_value(OfficeLocation)),
            "updated_at":
            lambda n: setattr(self, 'updated_at', n.get_datetime_value()),
            "birthday":
            lambda n: setattr(self, 'birthday', n.get_date_value()),
            "business_phones":
            lambda n: setattr(self, 'business_phones', n.get_collection_of_primitive_values(str)),
            "mobile_phone":
            lambda n: setattr(self, 'mobile_phone', n.get_str_value()),
            "is_active":
            lambda n: setattr(self, 'is_active', n.get_bool_value()),
        }

    def serialize(self, writer: SerializationWriter) -> None:
        """Writes the objects properties to the current writer.

        Args:
            writer (SerializationWriter): The writer to write to.
        """
        if not writer:
            raise TypeError("Writer cannot be null")
        writer.write_uuid_value("id", self.id)
        writer.write_enum_value("office_location", self.office_location)
        writer.write_datetime_value("updated_at", self.updated_at)
        writer.write_date_value("birthday", self.birthday)
        writer.write_collection_of_primitive_values("business_phones", self.business_phones)
        writer.write_str_value("mobile_phone", self.mobile_phone)
        writer.write_bool_value("is_active", self.is_active)



import json
from datetime import date, datetime, time, timedelta
from uuid import UUID
import pendulum

import pytest
from pendulum import DateTime, FixedTimezone
from kiota_serialization_json.json_parse_node import JsonParseNode
from ..helpers import OfficeLocation, User

url: str = "https://graph.microsoft.com/v1.0/$metadata#users/$entity"


def test_get_str_value():
    parse_node = JsonParseNode("Diego Siciliani")
    result = parse_node.get_str_value()
    assert result == "Diego Siciliani"


def test_get_int_value():
    parse_node = JsonParseNode(1454)
    result = parse_node.get_int_value()
    assert result == 1454


def test_get_bool_value():
    parse_node = JsonParseNode(False)
    result = parse_node.get_bool_value()
    assert result is False


def test_get_float_value_from_float():
    """
    This test is to ensure that the get_float_value method returns a float when the value is a float
    """
    parse_node = JsonParseNode(44.6)
    result = parse_node.get_float_value()
    assert isinstance(result, float)
    assert result == 44.6


@pytest.mark.parametrize("value", [0, 10, 100])
def test_get_float_value(value: int):
    """
    Consider an OpenAPI Specification using the type: number and format: float or double    
    Note: The OpenAPI Specification also allows for the use of the type: integer and format: int32 or int64

    Consider an API with Price data [0, 0.5, 1, 1.5, 2] and so on
    In this case, the contract must define the type as a number, with a hint of float or double as the format

    Kiota should be able to parse the response as a float, even if the value is an integer, because it's still a number.
    """
    parse_node = JsonParseNode(value)
    result = parse_node.get_float_value()
    assert isinstance(result, float)
    assert result == float(value)


def test_get_uuid_value():
    parse_node = JsonParseNode("f58411c7-ae78-4d3c-bb0d-3f24d948de41")
    result = parse_node.get_uuid_value()
    assert result == UUID("f58411c7-ae78-4d3c-bb0d-3f24d948de41")


@pytest.mark.parametrize("value", ["", " ", "  ", "2022-01-0"])
def test_get_datetime_value_returns_none_with_invalid_str(value: str):
    parse_node = JsonParseNode(value)
    result = parse_node.get_datetime_value()
    assert result is None


def test_get_datetime_value():
    parse_node = JsonParseNode('2022-01-27T12:59:45.596117')
    result = parse_node.get_datetime_value()
    assert isinstance(result, datetime)


def test_get_date_value():
    parse_node = JsonParseNode('2015-04-20')
    result = parse_node.get_date_value()
    assert isinstance(result, date)
    assert str(result) == '2015-04-20'


def test_get_time_value():
    parse_node = JsonParseNode('12:59:45.596117')
    result = parse_node.get_time_value()
    assert isinstance(result, time)
    assert str(result) == '12:59:45.596117'


def test_get_timedelta_value():
    parse_node = JsonParseNode('PT30S')
    result = parse_node.get_timedelta_value()
    assert isinstance(result, timedelta)
    assert str(result) == '0:00:30'


def test_get_collection_of_primitive_values():
    parse_node = JsonParseNode([12.1, 12.2, 12.3, 12.4, 12.5])
    result = parse_node.get_collection_of_primitive_values(float)
    assert result == [12.1, 12.2, 12.3, 12.4, 12.5]


def test_get_collection_of_primitive_values_no_type():
    parse_node = JsonParseNode(["One", "Two", "Three", "Four", "Five"])
    result = parse_node.get_collection_of_primitive_values(None)
    assert result == ["One", "Two", "Three", "Four", "Five"]


def test_get_bytes_value():
    parse_node = JsonParseNode('U2Ftd2VsIGlzIHRoZSBiZXN0')
    result = parse_node.get_bytes_value()
    assert isinstance(result, bytes)


def test_get_collection_of_enum_values():
    parse_node = JsonParseNode(["dunhill", "oval"])
    result = parse_node.get_collection_of_enum_values(OfficeLocation)
    assert isinstance(result, list)
    assert result == [OfficeLocation.Dunhill, OfficeLocation.Oval]


def test_get_enum_value():
    parse_node = JsonParseNode("dunhill")
    result = parse_node.get_enum_value(OfficeLocation)
    assert isinstance(result, OfficeLocation)
    assert result == OfficeLocation.Dunhill


def test_get_enum_value_for_key_not_found():
    parse_node = JsonParseNode("whitehouse")
    result = parse_node.get_enum_value(OfficeLocation)
    assert result is None


def test_get_anythin_does_not_convert_numeric_chars_to_datetime():
    parse_node = JsonParseNode("1212")
    result = parse_node.try_get_anything("1212")
    assert isinstance(result, str)
    assert result == "1212"


def test_get_anythin_does_convert_date_string_to_datetime():
    parse_node = JsonParseNode("2023-10-05T14:48:00.000Z")
    result = parse_node.try_get_anything("2023-10-05T14:48:00.000Z")
    assert isinstance(result, pendulum.DateTime)
    assert result == pendulum.parse("2023-10-05T14:48:00.000Z")


def test_get_object_value(user1_json):
    parse_node = JsonParseNode(json.loads(user1_json))
    result = parse_node.get_object_value(User)
    assert isinstance(result, User)
    assert result.id == UUID("8f841f30-e6e3-439a-a812-ebd369559c36")
    assert result.office_location == OfficeLocation.Dunhill
    assert isinstance(result.updated_at, datetime)
    assert isinstance(result.birthday, date)
    assert result.business_phones == ["+1 205 555 0108"]
    assert result.is_active is True
    assert result.mobile_phone is None
    assert result.additional_data["additional_data"][
        "@odata.context"] == "https://graph.microsoft.com/v1.0/$metadata#users/$entity"
    assert result.additional_data["additional_data"]["manager"] == {
        "id": UUID('8f841f30-e6e3-439a-a812-ebd369559c36'),
        "updated_at":
        DateTime(2022, 1, 27, 12, 59, 45, 596117, tzinfo=FixedTimezone(0, name="+00:00")),
        "is_active": True
    }
    assert result.additional_data["additional_data"]["approvers"] == [
        {
            "id":
            UUID('8f841f30-e6e3-439a-a812-ebd369559c36'),
            "updated_at":
            DateTime(2022, 1, 27, 12, 59, 45, 596117, tzinfo=FixedTimezone(0, name="+00:00")),
            "is_active":
            True
        }, {
            "display_name": "John Doe",
            "age": 32
        }
    ]
    assert result.additional_data["additional_data"]["data"] == {
        "groups": [{
            "friends": [{
                "display_name": "John Doe",
                "age": 32
            }]
        }]
    }


def test_get_collection_of_object_values(users_json):
    parse_node = JsonParseNode(json.loads(users_json))
    result = parse_node.get_collection_of_object_values(User)
    assert isinstance(result[0], User)
    assert result[0].id == UUID("8f841f30-e6e3-439a-a812-ebd369559c36")
    assert result[0].office_location == OfficeLocation.Dunhill
    assert isinstance(result[0].updated_at, datetime)
    assert isinstance(result[0].birthday, date)
    assert result[0].business_phones == ["+1 205 555 0108"]
    assert result[0].is_active is True
    assert result[0].mobile_phone is None



import json

import pytest

from kiota_serialization_json.json_parse_node import JsonParseNode
from kiota_serialization_json.json_parse_node_factory import JsonParseNodeFactory


@pytest.fixture
def sample_json_string():
    return '{"name":"Tesla", "age":2, "city":"New York"}'


def test_get_root_parse_node(sample_json_string):
    factory = JsonParseNodeFactory()
    sample_json_string_bytes = sample_json_string.encode('utf-8')
    root = factory.get_root_parse_node('application/json', sample_json_string_bytes)
    assert isinstance(root, JsonParseNode)
    assert root._json_node == json.loads(sample_json_string)

  
def test_get_root_parse_node_no_content_type(sample_json_string):
    with pytest.raises(Exception) as e_info:
        factory = JsonParseNodeFactory()
        sample_json_string_bytes = sample_json_string.encode('utf-8')
        root = factory.get_root_parse_node('', sample_json_string_bytes)


def test_get_root_parse_node_unsupported_content_type(sample_json_string):
    with pytest.raises(Exception) as e_info:
        factory = JsonParseNodeFactory()
        sample_json_string_bytes = sample_json_string.encode('utf-8')
        root = factory.get_root_parse_node('application/xml', sample_json_string_bytes)


def test_get_root_parse_node_empty_json():
    with pytest.raises(TypeError) as e_info:
        factory = JsonParseNodeFactory()
        sample_string_bytes = ''.encode('utf-8')
        root = factory.get_root_parse_node('application/json', sample_string_bytes)


def test_get_valid_content_type():
    factory = JsonParseNodeFactory()
    content_type = factory.get_valid_content_type()
    assert content_type == 'application/json'
